
### Sweeney, TE and Khatri, P
### Dec. 2015

### Questions: tes17 [at] stanford [dot] edu
### Website: khatrilab.stanford.edu/sepsis

### These functions are meant to be sourced by the main code file. 
### We do not expect users to need to open or read this source code.

scoreGeomMean <- function(genes, pos.genes, neg.genes){
    pos.genes <- genesCheck(genes, pos.genes)
    neg.genes <- genesCheck(genes, neg.genes)
    
    geomMeanPos <- apply(genes[pos.genes, , drop=F], 2, geomMean)
    geomMeanNeg <- apply(genes[neg.genes, , drop=F], 2, geomMean)
    
    ratio <- length(neg.genes)/length(pos.genes)
    score <- geomMeanPos - ratio * geomMeanNeg
    
    return(score)
}

geomMean <- function(vec){
    if(any(is.na(vec) | (vec < 0))) 
        stop("Cannot compute geometric mean for negative or missing values")
    prod(vec)^(1/length(vec))
}

scoreRatio <- function(genes, pos.genes, neg.genes){
    pos.genes <- genesCheck(genes, pos.genes)
    neg.genes <- genesCheck(genes, neg.genes)
    
    sumPos <- apply(genes[pos.genes, , drop=F], 2, sum)
    sumNeg <- apply(genes[neg.genes, , drop=F], 2, sum) 
    
    score <- sumPos/sumNeg
    
    return(score)
}

scoreSum <- function(genes, pos.genes, neg.genes){
    pos.genes <- genesCheck(genes, pos.genes)
    neg.genes <- genesCheck(genes, neg.genes)
    
    sumPos <- colSums(genes[pos.genes, , drop=F])
    sumNeg <- colSums(genes[neg.genes, , drop=F])
    
    score <- sumPos-sumNeg
    
    return(score)
}

## flag and remove missing genes
genesCheck <- function(genes, target){
    index <- target %in% rownames(genes)
    if(any(!index)) {
        cat(target[!index], " missing from this dataset and not used\n")
    }
    return(target[index])
}


## calculates the AUC for any two vectors. Removes NAs from both vectors.
## partially based on ROCR package
getAUC <- function(predictions, labels){
    NAs <- is.na(labels) | is.na(predictions)
    predictions <- predictions[!NAs]
    labels <- labels[!NAs]
    
    levels <- sort(unique(labels))
    labels <- ordered(labels, levels = levels)
    
    n.pos <- sum(labels == levels[2])
    n.neg <- sum(labels == levels[1])
    
    pred.order <- order(predictions, decreasing = TRUE)
    predictions.sorted <- predictions[pred.order]
    
    tp <- cumsum(labels[pred.order] == levels[2])
    fp <- cumsum(labels[pred.order] == levels[1])
    dups <- rev(duplicated(rev(predictions.sorted)))
    tp <- c(0, tp[!dups])
    fp <- c(0, fp[!dups])
    fn <- n.pos - tp
    tn <- n.neg - fp
    
    tpr <- tp / (tp + fn)
    fpr <- fp / (fp + tn)
    roc = data.frame(x = fpr, y = tpr)
    roc <- roc[order(roc$x, roc$y),]
    
    i <- 2:nrow(roc)
    auc <- (roc$x[i] - roc$x[i - 1]) %*% (roc$y[i] + roc$y[i - 1])/2
    
    auc <- signif(auc, 3)
    
    return(auc)
}
